@@ -15,16 +15,12 @@
 // commercial license or contractual agreement.
 
 
-#include <Adaptor2d_HCurve2d.hxx>
-#include <Adaptor3d_HSurface.hxx>
-#include <Adaptor3d_HVertex.hxx>
+#include <Adaptor2d_Curve2d.hxx>
 #include <Adaptor3d_TopolTool.hxx>
 #include <Blend_CurvPointFuncInv.hxx>
-#include <Blend_Point.hxx>
 #include <Blend_RstRstFunction.hxx>
 #include <Blend_SurfCurvFuncInv.hxx>
 #include <BRepBlend_BlendTool.hxx>
-#include <BRepBlend_Extremity.hxx>
 #include <BRepBlend_Line.hxx>
 #include <BRepBlend_RstRstLineBuilder.hxx>
 #include <gp_Pnt.hxx>
@@ -168,14 +164,19 @@ static void Drawsect(const Standard_Real param,
 //=======================================================================
 
 BRepBlend_RstRstLineBuilder::BRepBlend_RstRstLineBuilder
-(const Handle(Adaptor3d_HSurface)&  Surf1,
- const Handle(Adaptor2d_HCurve2d)&  Rst1,
+(const Handle(Adaptor3d_Surface)&  Surf1,
+ const Handle(Adaptor2d_Curve2d)&  Rst1,
  const Handle(Adaptor3d_TopolTool)& Domain1,
- const Handle(Adaptor3d_HSurface)&  Surf2,
- const Handle(Adaptor2d_HCurve2d)&  Rst2,
+ const Handle(Adaptor3d_Surface)&  Surf2,
+ const Handle(Adaptor2d_Curve2d)&  Rst2,
  const Handle(Adaptor3d_TopolTool)& Domain2):
- sol(1,2), surf1(Surf1), domain1(Domain1),
- surf2(Surf2), domain2(Domain2), rst1(Rst1), rst2(Rst2)
+ done(Standard_False), sol(1, 2), surf1(Surf1),
+ domain1(Domain1), surf2(Surf2),
+ domain2(Domain2), rst1(Rst1), rst2(Rst2),
+ tolesp(0.0), tolgui(0.0), pasmax(0.0),
+ fleche(0.0), param(0.0), rebrou(Standard_False),
+ iscomplete(Standard_False), comptra(Standard_False), sens(0.0),
+ decrochdeb(Blend_NoDecroch), decrochfin(Blend_NoDecroch)
 {
 }
 
@@ -322,7 +323,7 @@ Standard_Boolean BRepBlend_RstRstLineBuilder::PerformFirstSection
   math_Vector solinvp1(1, 2), solinvp2(1, 2), solinvrst1(1, 3), solinvrst2(1, 3);
   Handle(Adaptor3d_HVertex) Vtxp1, Vtxp2, Vtxrst1, Vtxrst2, Vtxc;
   Standard_Boolean IsVtxp1 = 0, IsVtxp2 = 0, IsVtxrst1 = 0, IsVtxrst2 = 0;
-  Handle(Adaptor2d_HCurve2d) Arc;
+  Handle(Adaptor2d_Curve2d) Arc;
   wp1   = wp2 = wrst1 = wrst2 = Pmax;
   param = Pdep;
   Func.Set(param);
@@ -1316,7 +1317,7 @@ Standard_Boolean BRepBlend_RstRstLineBuilder::Recadre2(Blend_CurvPointFuncInv&
 //=======================================================================
 
 void BRepBlend_RstRstLineBuilder::Transition(const Standard_Boolean          OnFirst,
-                                             const Handle(Adaptor2d_HCurve2d)& Arc,
+                                             const Handle(Adaptor2d_Curve2d)& Arc,
 					     const Standard_Real             Param,
 					     IntSurf_Transition&             TLine,
 					     IntSurf_Transition&             TArc) 
@@ -1366,7 +1367,7 @@ void BRepBlend_RstRstLineBuilder::Transition(const Standard_Boolean          OnF
 
 void BRepBlend_RstRstLineBuilder::MakeExtremity(BRepBlend_Extremity&            Extrem,
 						const Standard_Boolean          OnFirst,
-					        const Handle(Adaptor2d_HCurve2d)& Arc,
+					        const Handle(Adaptor2d_Curve2d)& Arc,
 					        const Standard_Real             Param,
 					        const Standard_Boolean          IsVtx,
 					        const Handle(Adaptor3d_HVertex)&  Vtx) 
@@ -1399,7 +1400,7 @@ void BRepBlend_RstRstLineBuilder::MakeExtremity(BRepBlend_Extremity&
   else {
     Extrem.SetVertex(Vtx);
     while (Iter->More()) {
-      Handle(Adaptor2d_HCurve2d) arc = Iter->Value();
+      Handle(Adaptor2d_Curve2d) arc = Iter->Value();
      if (arc != Arc) {
 	Iter->Initialize(arc);
 	Iter->InitVertexIterator();
