@@ -17,14 +17,13 @@
 
 #include <BRep_Builder.hxx>
 #include <BRep_Tool.hxx>
-#include <BRepAdaptor_HCompCurve.hxx>
+#include <BRepAdaptor_CompCurve.hxx>
 #include <BRepBuilderAPI_Copy.hxx>
 #include <BRepBuilderAPI_Transform.hxx>
 #include <BRepClass3d_SolidClassifier.hxx>
 #include <BRepFill.hxx>
 #include <BRepFill_ACRLaw.hxx>
 #include <BRepFill_CompatibleWires.hxx>
-#include <BRepFill_DataMapOfShapeHArray2OfShape.hxx>
 #include <BRepFill_Edge3DLaw.hxx>
 #include <BRepFill_EdgeOnSurfLaw.hxx>
 #include <BRepFill_LocationLaw.hxx>
@@ -36,10 +35,8 @@
 #include <BRepFill_ShapeLaw.hxx>
 #include <BRepFill_Sweep.hxx>
 #include <BRepGProp.hxx>
-#include <BRepLib_MakeEdge.hxx>
 #include <BRepLib_MakeFace.hxx>
-#include <GeomAdaptor_HCurve.hxx>
-#include <GeomAdaptor_HSurface.hxx>
+#include <GeomAdaptor_Surface.hxx>
 #include <GeomFill_ConstantBiNormal.hxx>
 #include <GeomFill_CorrectedFrenet.hxx>
 #include <GeomFill_CurveAndTrihedron.hxx>
@@ -58,19 +55,15 @@
 #include <GProp_GProps.hxx>
 #include <IntCurveSurface_HInter.hxx>
 #include <IntCurveSurface_IntersectionPoint.hxx>
-#include <Law_Function.hxx>
 #include <Law_Interpol.hxx>
 #include <Precision.hxx>
 #include <Standard_ConstructionError.hxx>
-#include <Standard_DomainError.hxx>
 #include <Standard_NotImplemented.hxx>
 #include <Standard_Type.hxx>
 #include <StdFail_NotDone.hxx>
 #include <TColgp_HArray1OfPnt2d.hxx>
-#include <TColStd_HArray1OfReal.hxx>
 #include <TColStd_SequenceOfInteger.hxx>
 #include <TopExp.hxx>
-#include <TopLoc_Location.hxx>
 #include <TopoDS.hxx>
 #include <TopoDS_Iterator.hxx>
 #include <TopoDS_Shape.hxx>
@@ -91,10 +84,8 @@ IMPLEMENT_STANDARD_RTTIEXT(BRepFill_PipeShell,Standard_Transient)
 static Standard_Boolean Affich = 0;
 #endif
 
-#include <TopTools_ListIteratorOfListOfShape.hxx>
 #include <TopTools_IndexedDataMapOfShapeListOfShape.hxx>
 #include <TopTools_DataMapOfIntegerShape.hxx>
-#include <TopoDS_Compound.hxx>
 
 
 static Standard_Boolean BuildBoundaries(const BRepFill_Sweep&               theSweep,
@@ -386,7 +377,7 @@ BRepFill_PipeShell::BRepFill_PipeShell(const TopoDS_Wire& Spine)
     DBRep::Set("theguide", TheGuide);
 #endif
   // transform the guide in a single curve
-  Handle(BRepAdaptor_HCompCurve) Guide = new (BRepAdaptor_HCompCurve) (TheGuide);
+  Handle(BRepAdaptor_CompCurve) Guide = new (BRepAdaptor_CompCurve) (TheGuide);
 
   if (CurvilinearEquivalence) { // trihedron by curvilinear reduced abscissa
     if (KeepContact == BRepFill_Contact ||
@@ -491,9 +482,9 @@ void BRepFill_PipeShell::SetForceApproxC1(const Standard_Boolean ForceApproxC1)
 
    TopoDS_Face ProfileFace = BRepLib_MakeFace(TopoDS::Wire(Profile), Standard_True); //only plane
    Handle(Geom_Surface) thePlane = BRep_Tool::Surface(ProfileFace);
-   Handle(GeomAdaptor_HSurface) GAHplane = new GeomAdaptor_HSurface(thePlane);
+   Handle(GeomAdaptor_Surface) GAHplane = new GeomAdaptor_Surface(thePlane);
    IntCurveSurface_HInter Intersector;
-   Handle(Adaptor3d_HCurve) aHCurve [2];
+   Handle(Adaptor3d_Curve) aHCurve [2];
    aHCurve[0] = Loc->GetCurve();
    aHCurve[1] = Loc->Guide();
    gp_Pnt PointsOnSpines [2];
@@ -732,7 +723,7 @@ void BRepFill_PipeShell::SetForceApproxC1(const Standard_Boolean ForceApproxC1)
     }
     // eap 5 Jun 2002 occ332, end modif
   }
-#if DRAW
+#ifdef DRAW
   if (Affich) {
     DBRep::Set("PipeFirst", myFirst);
     DBRep::Set("PipeLast",  myLast);
@@ -1377,7 +1368,7 @@ void BRepFill_PipeShell::BuildHistory(const BRepFill_Sweep& theSweep)
           BB.Add(aWire, CurEdge);
         } //for (jj = 2; jj <= SeqEdges.Length(); jj++)
         //case of closed wire
-        if (mySection->IsVClosed() &&
+        if (myLocation->IsClosed() &&
             !CurVertex.IsSame(FirstVertex))
         {
           const TopTools_ListOfShape& Elist = VEmap.FindFromKey(CurVertex);
@@ -1450,6 +1441,10 @@ void BRepFill_PipeShell::BuildHistory(const BRepFill_Sweep& theSweep)
     for (Standard_Integer i = 1; i <= aVEdges->UpperRow(); i++)
     {
       const TopoDS_Shape& aVshape = aVEdges->Value(i, inde);
+      if (aVshape.IsNull())
+      {
+        continue;
+      }
       if (aVshape.ShapeType() == TopAbs_EDGE ||
           aVshape.ShapeType() == TopAbs_FACE)
         ListVshapes->Append(aVshape);
