@@ -23,7 +23,6 @@
 #include <Draw_Appli.hxx>
 
 #include <BRepFill.hxx>
-#include <BRepBuilderAPI_PipeError.hxx>
 #include <BRepFill_Generator.hxx>
 #include <BRepPrimAPI_MakePrism.hxx>
 #include <BRepPrimAPI_MakeRevol.hxx>
@@ -35,13 +34,11 @@
 
 #include <BRepLib_MakeWire.hxx>
 #include <TopoDS.hxx>
-#include <TopTools_ListIteratorOfListOfShape.hxx>
 #include <TopExp_Explorer.hxx>
 
 #include <Precision.hxx>
 #include <Law_Interpol.hxx>
 #include <gp_Ax1.hxx>
-#include <gp_Ax2.hxx>
 #include <gp_Pnt2d.hxx>
 #include <TColgp_Array1OfPnt2d.hxx>
 
@@ -50,7 +47,7 @@ static BRepOffsetAPI_ThruSections* Generator = 0;
 
 #include <stdio.h>
 #include <Geom_Curve.hxx>
-#include <GeomAdaptor_HCurve.hxx>
+#include <GeomAdaptor_Curve.hxx>
 #include <GeomFill_Pipe.hxx>
 #include <Geom_Surface.hxx>
 #include <BRepBuilderAPI_MakeFace.hxx>
@@ -59,7 +56,8 @@ static BRepOffsetAPI_ThruSections* Generator = 0;
 #include <gp_Vec.hxx>
 #include <Geom_Circle.hxx>
 #include <gp_Ax2.hxx>
-
+#include <Message.hxx>
+#include <ShapeUpgrade_UnifySameDomain.hxx>
 
 //=======================================================================
 // prism
@@ -225,7 +223,7 @@ static Standard_Integer geompipe(Draw_Interpretor&,
   Standard_Real aSpFirst, aSpLast, aPrFirst, aPrLast;
   Handle(Geom_Curve) SpineCurve = BRep_Tool::Curve(TopoDS::Edge(Spine), aSpFirst, aSpLast);
   Handle(Geom_Curve) ProfileCurve = BRep_Tool::Curve(TopoDS::Edge(Profile), aPrFirst, aPrLast);
-  Handle(GeomAdaptor_HCurve) aAdaptCurve = new GeomAdaptor_HCurve(SpineCurve, aSpFirst, aSpLast);
+  Handle(GeomAdaptor_Curve) aAdaptCurve = new GeomAdaptor_Curve(SpineCurve, aSpFirst, aSpLast);
   Standard_Boolean ByACR = Standard_False;
   Standard_Boolean rotate = Standard_False;
   Standard_Real Radius = Draw::Atof(a[4]);
@@ -244,9 +242,13 @@ static Standard_Integer geompipe(Draw_Interpretor&,
   aPipe.Perform(Standard_True);
   if (!aPipe.IsDone())
   {
-    std::cout << "GeomFill_Pipe cannot make a surface" << std::endl;
+    Message::SendFail() << "GeomFill_Pipe cannot make a surface";
     return 1;
   }
+
+  Standard_Real Accuracy = aPipe.ErrorOnSurf();
+  std::cout << "Accuracy of approximation = " << Accuracy << std::endl;
+  
   Handle(Geom_Surface) Sur = aPipe.Surface();
   TopoDS_Face F;
   if (!Sur.IsNull())
@@ -267,7 +269,7 @@ Standard_Integer evolved(Draw_Interpretor& di, Standard_Integer n, const char**
     di << "   Make evolved profile on spine.\n";
     di << "   -solid means make closed solid.\n";
     di << "   -v means use alternative algorithm (volume mode).\n";
-    di << "   -a means referencial CS is automatically computed, otherwise global CS is used. \n";
+    di << "   -a means referential CS is automatically computed, otherwise global CS is used. \n";
     di << "   -t sets the tolerance.\n";
     di << "   -parallel turns on parallel execution.\n";
     return 0;
@@ -466,10 +468,15 @@ Standard_Integer thrusections(Draw_Interpretor&, Standard_Integer n, const char*
     Generator = 0;
   }
   Generator = new BRepOffsetAPI_ThruSections(issolid, isruled);
-
+  Standard_Boolean IsMutableInput = Standard_True;
   Standard_Integer NbEdges = 0;
   Standard_Boolean IsFirstWire = Standard_False;
   for (Standard_Integer i = index + 2; i <= n - 1; i++) {
+    if (!strcmp(a[i], "-safe"))
+    {
+      IsMutableInput = Standard_False;
+      continue;
+    }
     Standard_Boolean IsWire = Standard_True;
     Shape = DBRep::Get(a[i], TopAbs_WIRE);
     if (!Shape.IsNull())
@@ -503,6 +510,8 @@ Standard_Integer thrusections(Draw_Interpretor&, Standard_Integer n, const char*
 
   }
 
+  Generator->SetMutableInput(IsMutableInput);
+
   check = (check || !samenumber);
   Generator->CheckCompatibility(check);
 
@@ -525,16 +534,50 @@ Standard_Integer thrusections(Draw_Interpretor&, Standard_Integer n, const char*
 //=======================================================================
 //  mksweep
 //=======================================================================
-static Standard_Integer mksweep(Draw_Interpretor&,
+static Standard_Integer mksweep(Draw_Interpretor& di,
   Standard_Integer n, const char** a)
 {
-  if (n != 2) return 1;
+  if (n != 2 && n != 5) return 1;
   TopoDS_Shape Spine = DBRep::Get(a[1], TopAbs_WIRE);
   if (Spine.IsNull()) return 1;
   if (Sweep != 0)  {
     delete Sweep;
     Sweep = 0;
   }
+
+  if (n > 2 && n <= 5)
+  {
+    if (!strcmp(a[2], "-C"))
+    {
+      ShapeUpgrade_UnifySameDomain aUnif(Spine, Standard_True, Standard_False, Standard_True);
+
+      Standard_Real anAngTol = 5.;
+      Standard_Real aLinTol = 0.1;
+
+      if (n == 5)
+      {
+        anAngTol = Draw::Atof(a[3]);
+        aLinTol = Draw::Atof(a[4]);
+      }
+
+      aUnif.SetAngularTolerance(anAngTol * M_PI / 180.);
+      aUnif.SetLinearTolerance(aLinTol);
+      aUnif.Build();
+      Spine = aUnif.Shape();
+      if (BRepTest_Objects::IsHistoryNeeded())
+      {
+        BRepTest_Objects::SetHistory(aUnif.History());
+      }
+    }
+    else
+    {
+      di << "To correct input spine use 'mksweep wire -C [AngTol LinTol]'\n";
+      di << "By default, AngTol = 5, LinTol = 0.1";
+      return 1;
+    }
+  }
+
+
   Sweep = new BRepOffsetAPI_MakePipeShell(TopoDS::Wire(Spine));
   return 0;
 }
@@ -546,16 +589,6 @@ static Standard_Integer setsweep(Draw_Interpretor& di,
   Standard_Integer n, const char** a)
 {
   if (n == 1) {
-    //std::cout << "setsweep options [arg1 [arg2 [...]]] : options are :" << std::endl;
-    //std::cout << "   -FR : Tangent and Normal are given by Frenet trihedron" <<std::endl;
-    //std::cout << "   -CF : Tangente is given by Frenet," << std::endl;
-    //std::cout << "         the Normal is computed to minimize the torsion " << std::endl;
-    //std::cout << "   -DX Surf : Tangent and Normal are given by Darboux trihedron,"
-    //  <<std::endl;     
-    //std::cout << "       Surf have to be a shell or a face" <<std::endl;
-    //std::cout << "   -CN dx dy dz : BiNormal is given by dx dy dz" << std::endl;
-    //std::cout << "   -FX Tx Ty TZ [Nx Ny Nz] : Tangent and Normal are fixed" <<std::endl;
-    //std::cout << "   -G guide  0|1(ACR|Plan)  0|1(contact|no contact) : with guide"<<std::endl;
     di << "setsweep options [arg1 [arg2 [...]]] : options are :\n";
     di << "   -FR : Tangent and Normal are given by Frenet trihedron\n";
     di << "   -CF : Tangente is given by Frenet,\n";
@@ -566,11 +599,14 @@ static Standard_Integer setsweep(Draw_Interpretor& di,
     di << "   -CN dx dy dz : BiNormal is given by dx dy dz\n";
     di << "   -FX Tx Ty TZ [Nx Ny Nz] : Tangent and Normal are fixed\n";
     di << "   -G guide  0|1(Plan|ACR)  0|1|2(no contact|contact|contact on border) : with guide\n";
+    di << "   -SM : Set the maximum degree of approximation\n";
+    di << "         paramvalue more then 0 (100 by default)\n";
+    di << "   -DM : Set the maximum number of span of approximation\n";
+    di << "         paramvalue [1; 14] (11 by default)\n";
     return 0;
   }
 
   if (Sweep == 0) {
-    //std::cout << "You have forgotten the <<mksweep>> command  !"<< std::endl;
     di << "You have forgotten the <<mksweep>> command  !\n";
     return 1;
   }
@@ -585,14 +621,12 @@ static Standard_Integer setsweep(Draw_Interpretor& di,
   }
   else if (!strcmp(a[1], "-DX")) {
     if (n != 3) {
-      //std::cout << "bad arguments !" << std::endl;
       di << "bad arguments !\n";
       return 1;
     }
     TopoDS_Shape Surf;
     Surf = DBRep::Get(a[2], TopAbs_SHAPE);
     if (Surf.IsNull()) {
-      //std::cout << a[2] <<"is not a shape !" << std::endl;
       di << a[2] << "is not a shape !\n";
       return 1;
     }
@@ -600,16 +634,14 @@ static Standard_Integer setsweep(Draw_Interpretor& di,
   }
   else if (!strcmp(a[1], "-CN")) {
     if (n != 5) {
-      //std::cout << "bad arguments !" << std::endl;
       di << "bad arguments !\n";
       return 1;
     }
     gp_Dir D(Draw::Atof(a[2]), Draw::Atof(a[3]), Draw::Atof(a[4]));
-    Sweep->SetMode(D);;
+    Sweep->SetMode(D);
   }
   else if (!strcmp(a[1], "-FX")) {
     if ((n != 5) && (n != 8)) {
-      //std::cout << "bad arguments !" << std::endl;
       di << "bad arguments !\n";
       return 1;
     }
@@ -628,7 +660,6 @@ static Standard_Integer setsweep(Draw_Interpretor& di,
   {
     if (n != 5)
     {
-      //std::cout << "bad arguments !" << std::endl;
       di << "bad arguments !\n";
       return 1;
     }
@@ -642,16 +673,50 @@ static Standard_Integer setsweep(Draw_Interpretor& di,
         (BRepFill_TypeOfContact)KeepContact);
     }
   }
+  else if (!strcmp(a[1], "-DM"))
+  {
+    if (n != 3)
+    {
+      di << "bad arguments !\n";
+      return 1;
+    }
+
+    if (Draw::Atoi(a[2]) > 0 && Draw::Atoi(a[2]) < 15)
+    {
+      Sweep->SetMaxDegree(Draw::Atoi(a[2]));
+    }
+    else
+    {
+      di << " -DM paramvalue must be [1; 14]\n";
+      return 1;
+    }
+  }
+  else if (!strcmp(a[1], "-SM"))
+  {
+    if (n != 3)
+    {
+      di << "bad arguments !\n";
+      return 1;
+    }
+
+    if (Draw::Atoi(a[2]) > 0)
+    {
+      Sweep->SetMaxSegments(Draw::Atoi(a[2]));
+    }
+    else
+    {
+      di << " -SM paramvalue must be more then 0\n";
+      return 1;
+    }
+  }
 
   else {
-    //std::cout << "The option "<< a[1] << " is unknown !" << std::endl;
     di << "The option " << a[1] << " is unknown !\n";
     return 1;
   }
   return 0;
 }
 
-
 //=======================================================================
 //  addsweep
 //=======================================================================
@@ -659,11 +724,6 @@ static Standard_Integer addsweep(Draw_Interpretor& di,
   Standard_Integer n, const char** a)
 {
   if (n == 1) {
-    //std::cout << "addsweep wire/vertex [Vertex] [-T] [-R] [u0 v0 u1 v1 [...[uN vN]]] : options are :" << std::endl;
-    //std::cout << "   -T : the wire/vertex have to be translated to assume contact"<< std::endl;
-    //std::cout << "        with the spine" <<std::endl;
-    //std::cout << "   -R : the wire have to be rotated to assume orthogonality"<<std::endl;
-    //std::cout << "        with the spine's tangent" << std::endl;
     di << "addsweep wire/vertex [Vertex] [-T] [-R] [u0 v0 u1 v1 [...[uN vN]]] : options are :\n";
     di << "   -T : the wire/vertex have to be translated to assume contact\n";
     di << "        with the spine\n";
@@ -673,7 +733,6 @@ static Standard_Integer addsweep(Draw_Interpretor& di,
   }
 
   if (Sweep == 0) {
-    //std::cout << "You have forgotten the <<mksweep>> command  !"<< std::endl;
     di << "You have forgotten the <<mksweep>> command  !\n";
     return 1;
   }
@@ -687,7 +746,6 @@ static Standard_Integer addsweep(Draw_Interpretor& di,
     (Section.ShapeType() != TopAbs_WIRE &&
     Section.ShapeType() != TopAbs_VERTEX))
   {
-    //std::cout << a[1] <<"is not a wire and is not a vertex!" << std::endl;
     di << a[1] << " is not a wire and is not a vertex!\n";
     return 1;
   }
@@ -701,7 +759,6 @@ static Standard_Integer addsweep(Draw_Interpretor& di,
     // Reading of Vertex
     TopoDS_Shape InputVertex(DBRep::Get(a[cur], TopAbs_VERTEX));
     Vertex = TopoDS::Vertex(InputVertex);
-    //    Vertex = TopoDS::Vertex(DBRep::Get(a[cur],TopAbs_VERTEX));
     if (!Vertex.IsNull()) {
       cur++;
       HasVertex = Standard_True;
@@ -764,15 +821,12 @@ static Standard_Integer deletesweep(Draw_Interpretor& di,
   TopoDS_Wire Section;
   TopoDS_Shape InputShape(DBRep::Get(a[1], TopAbs_SHAPE));
   Section = TopoDS::Wire(InputShape);
-  //  Section = TopoDS::Wire(DBRep::Get(a[1],TopAbs_SHAPE));
   if (Section.IsNull()) {
-    //std::cout << a[1] <<"is not a wire !" << std::endl;
     di << a[1] << "is not a wire !\n";
     return 1;
   }
 
   Sweep->Delete(Section);
-
   return 0;
 }
 
@@ -783,34 +837,24 @@ static Standard_Integer buildsweep(Draw_Interpretor& di,
   Standard_Integer n, const char** a)
 {
   if (n == 1) {
-    //std::cout << "build sweep result [-M/-C/-R] [-S] [tol] : options are" << std::endl;
-    //std::cout << "   -M : Discontinuities are treated by Modfication of"<< std::endl; 
-    //std::cout << "        the sweeping mode : it is the default" <<std::endl;
-    //std::cout << "   -C : Discontinuities are treated like Right Corner" << std::endl;
-    //std::cout << "        Treatement is Extent && Intersect" << std::endl;
-    //std::cout << "   -R : Discontinuities are treated like Round Corner" << std::endl;
-    //std::cout << "        Treatement is Intersect and Fill" << std::endl;
-    //std::cout << "   -S : To build a Solid" << std::endl;
     di << "build sweep result [-M/-C/-R] [-S] [tol] : options are\n";
-    di << "   -M : Discontinuities are treated by Modfication of\n";
+    di << "   -M : Discontinuities are treated by Modification of\n";
     di << "        the sweeping mode : it is the default\n";
     di << "   -C : Discontinuities are treated like Right Corner\n";
-    di << "        Treatement is Extent && Intersect\n";
+    di << "        Treatment is Extent && Intersect\n";
     di << "   -R : Discontinuities are treated like Round Corner\n";
-    di << "        Treatement is Intersect and Fill\n";
+    di << "        Treatment is Intersect and Fill\n";
     di << "   -S : To build a Solid\n";
     return 0;
   }
 
   Standard_Boolean mksolid = Standard_False;
   if (Sweep == 0) {
-    //std::cout << "You have forgotten the <<mksweep>> command  !"<< std::endl;
     di << "You have forgotten the <<mksweep>> command  !\n";
     return 1;
   }
 
   if (!Sweep->IsReady()) {
-    //std::cout << "You have forgotten the <<addsweep>> command  !"<< std::endl;
     di << "You have forgotten the <<addsweep>> command  !\n";
     return 1;
   }
@@ -837,15 +881,12 @@ static Standard_Integer buildsweep(Draw_Interpretor& di,
   // Calcul le resultat
   Sweep->Build();
   if (!Sweep->IsDone()) {
-    //std::cout << "Buildsweep : Not Done" << std::endl;
     di << "Buildsweep : Not Done\n";
     BRepBuilderAPI_PipeError Stat = Sweep->GetStatus();
     if (Stat == BRepBuilderAPI_PlaneNotIntersectGuide) {
-      //std::cout << "Buildsweep : One Plane not intersect the guide" << std::endl;
       di << "Buildsweep : One Plane not intersect the guide\n";
     }
     if (Stat == BRepBuilderAPI_ImpossibleContact) {
-      //std::cout << "BuildSweep : One section can not be in contact with the guide" << std::endl;
       di << "BuildSweep : One section can not be in contact with the guide\n";
     }
   }
@@ -853,7 +894,6 @@ static Standard_Integer buildsweep(Draw_Interpretor& di,
     if (mksolid) {
       Standard_Boolean B;
       B = Sweep->MakeSolid();
-      //if (!B) std::cout << " BuildSweep : It is impossible to make a solid !" << std::endl;
       if (!B) di << " BuildSweep : It is impossible to make a solid !\n";
     }
     result = Sweep->Shape();
@@ -899,13 +939,11 @@ static Standard_Integer simulsweep(Draw_Interpretor& di,
   if ((n != 3) && (n != 4)) return 1;
 
   if (Sweep == 0) {
-    //std::cout << "You have forgotten the <<mksweep>> command  !"<< std::endl;
     di << "You have forgotten the <<mksweep>> command  !\n";
     return 1;
   }
 
   if (!Sweep->IsReady()) {
-    //std::cout << "You have forgotten the <<addsweep>> command  !"<< std::endl;
     di << "You have forgotten the <<addsweep>> command  !\n";
     return 1;
   }
@@ -1002,10 +1040,14 @@ void  BRepTest::SweepCommands(Draw_Interpretor& theCommands)
   theCommands.Add("gener", "gener result wire1 wire2 [..wire..]",
     __FILE__, gener, g);
 
-  theCommands.Add("thrusections", "thrusections [-N] result issolid isruled shape1 shape2 [..shape..], the option -N means no check on wires, shapes must be wires or vertices (only first or last)",
+  theCommands.Add("thrusections", "thrusections [-N] result issolid isruled shape1 shape2 [..shape..] [-safe],\n"
+    "\t\tthe option -N means no check on wires, shapes must be wires or vertices (only first or last),\n"
+    "\t\t-safe option allows to prevent the modifying of input shapes",
     __FILE__, thrusections, g);
 
-  theCommands.Add("mksweep", "mksweep wire",
+  theCommands.Add("mksweep", "mksweep wire [-C [AngTol LinTol]]\n"
+    "\t\tthe option -C correct input spine by merging smooth connected neighboring edges\n"
+    "\t\tthe AngTol is in degrees",
     __FILE__, mksweep, g);
 
   theCommands.Add("setsweep", "setsweep  no args to get help",
@@ -1025,9 +1067,9 @@ void  BRepTest::SweepCommands(Draw_Interpretor& theCommands)
   theCommands.Add("errorsweep", "errorsweep: returns the summary error on resulting surfaces reached by Sweep",
     __FILE__, errorsweep, g);
 
-  theCommands.Add("simulsweep", "simulsweep r [n] [option]"
+  theCommands.Add("simulsweep", "simulsweep r [n] [option]",
     __FILE__, simulsweep, g);
-  theCommands.Add("geompipe", "geompipe r spineedge profileedge radius [byACR [byrotate]]"
+  theCommands.Add("geompipe", "geompipe r spineedge profileedge radius [byACR [byrotate]]",
     __FILE__, geompipe, g);
 
   theCommands.Add("middlepath", "middlepath res shape startshape endshape",
