@@ -15,65 +15,37 @@
 // commercial license or contractual agreement.
 
 
-#include <Adaptor2d_HCurve2d.hxx>
-#include <Adaptor3d_HSurface.hxx>
-#include <Adaptor3d_TopolTool.hxx>
-#include <AppBlend_Approx.hxx>
+#include <Adaptor2d_Curve2d.hxx>
 #include <Blend_CurvPointFuncInv.hxx>
 #include <Blend_FuncInv.hxx>
-#include <Blend_Function.hxx>
-#include <Blend_RstRstFunction.hxx>
-#include <Blend_SurfCurvFuncInv.hxx>
-#include <Blend_SurfPointFuncInv.hxx>
-#include <Blend_SurfRstFunction.hxx>
-#include <BRep_Tool.hxx>
-#include <BRepAdaptor_Curve2d.hxx>
-#include <BRepAdaptor_HCurve.hxx>
-#include <BRepAdaptor_HCurve2d.hxx>
-#include <BRepAdaptor_HSurface.hxx>
 #include <BRepBlend_Line.hxx>
 #include <BRepTopAdaptor_TopolTool.hxx>
 #include <ChFi3d_Builder.hxx>
 #include <ChFi3d_Builder_0.hxx>
 #include <ChFiDS_CommonPoint.hxx>
-#include <ChFiDS_HElSpine.hxx>
 #include <ChFiDS_Spine.hxx>
-#include <ChFiDS_Stripe.hxx>
 #include <ChFiDS_SurfData.hxx>
 #include <ChFiKPart_RstMap.hxx>
 #include <ElCLib.hxx>
 #include <ElSLib.hxx>
-#include <Geom2d_BezierCurve.hxx>
-#include <Geom2d_BSplineCurve.hxx>
 #include <Geom2d_Curve.hxx>
-#include <Geom2d_Line.hxx>
 #include <Geom2dAdaptor_Curve.hxx>
-#include <Geom2dAdaptor_HCurve.hxx>
 #include <Geom2dHatch_Hatcher.hxx>
 #include <Geom2dHatch_Intersector.hxx>
 #include <Geom_Curve.hxx>
 #include <Geom_Plane.hxx>
 #include <Geom_Surface.hxx>
 #include <GeomAbs_CurveType.hxx>
-#include <GeomAbs_SurfaceType.hxx>
-#include <GeomAdaptor_Curve.hxx>
-#include <GeomAdaptor_HSurface.hxx>
-#include <gp.hxx>
+#include <GeomAdaptor_Surface.hxx>
 #include <gp_Circ.hxx>
-#include <gp_Dir2d.hxx>
 #include <gp_Pnt.hxx>
 #include <gp_Pnt2d.hxx>
 #include <HatchGen_Domain.hxx>
 #include <HatchGen_PointOnElement.hxx>
 #include <HatchGen_PointOnHatching.hxx>
 #include <Precision.hxx>
-#include <Standard_ConstructionError.hxx>
-#include <Standard_NoSuchObject.hxx>
 #include <Standard_NotImplemented.hxx>
-#include <Standard_OutOfRange.hxx>
-#include <TColgp_Array1OfPnt2d.hxx>
 #include <TColStd_Array1OfInteger.hxx>
-#include <TColStd_Array1OfReal.hxx>
 #include <TColStd_SequenceOfInteger.hxx>
 #include <TopExp.hxx>
 #include <TopoDS_Edge.hxx>
@@ -260,8 +232,8 @@ static Standard_Real ParamOnSpine(const TopOpeBRepDS_DataStructure& DStr,
     
     gp_Pln nlp(PP,gp_Dir(VV));
     Handle(Geom_Plane) pln = new Geom_Plane(nlp);
-    Handle(GeomAdaptor_HSurface) 
-      plan = new GeomAdaptor_HSurface(GeomAdaptor_Surface(pln));
+    Handle(GeomAdaptor_Surface) 
+      plan = new GeomAdaptor_Surface(GeomAdaptor_Surface(pln));
     
     // intersection plane spine.
     Standard_Boolean found = Standard_False;
@@ -277,8 +249,8 @@ static Standard_Real ParamOnSpine(const TopOpeBRepDS_DataStructure& DStr,
       pok = 1;
       return Nl;
     }
-    Handle(BRepAdaptor_HCurve) HE = new BRepAdaptor_HCurve();
-    BRepAdaptor_Curve& CE = HE->ChangeCurve();
+    Handle(BRepAdaptor_Curve) HE = new BRepAdaptor_Curve();
+    BRepAdaptor_Curve& CE = *HE;
     
     while (!found && !fini) {
       TopAbs_Orientation O = Spine->Edges(ii).Orientation();
@@ -289,17 +261,17 @@ static Standard_Real ParamOnSpine(const TopOpeBRepDS_DataStructure& DStr,
       found = ChFi3d_InterPlaneEdge(plan,HE,Nl,First,tolc);
       gp_Pnt point = CE.Value(Nl);
 #ifdef OCCT_DEBUG
-      cout<<"******* ParamOnSpine() for edge "<<iedge<<endl;
-      cout<<Nl<<endl;
-      cout<<"point ped "<<point.X()<<" "<<point.Y()<<" "<<point.Z()<<endl;
+      std::cout<<"******* ParamOnSpine() for edge "<<iedge<<std::endl;
+      std::cout<<Nl<<std::endl;
+      std::cout<<"point ped "<<point.X()<<" "<<point.Y()<<" "<<point.Z()<<std::endl;
 #endif
       if(found) Nl = Spine->Absc(Nl,ii);
       point = Spine->Value(Nl);
 #ifdef OCCT_DEBUG
-      if (found) cout << "found by edge " << ii << " : ";
-      cout<<Nl<<endl;
-      cout<<"point psp "<<point.X()<<" "<<point.Y()<<" "<<point.Z()<<endl;
-      cout<<endl;
+      if (found) std::cout << "found by edge " << ii << " : ";
+      std::cout<<Nl<<std::endl;
+      std::cout<<"point psp "<<point.X()<<" "<<point.Y()<<" "<<point.Z()<<std::endl;
+      std::cout<<std::endl;
 #endif
       
       ii +=sens;
@@ -346,8 +318,8 @@ static Standard_Boolean YaUnVoisin(const Handle(ChFiDS_Spine)& Spine,
 
 void ChFi3d_Builder::Trunc(const Handle(ChFiDS_SurfData)&    SD,
 			   const Handle(ChFiDS_Spine)&       Spine,
-			   const Handle(Adaptor3d_HSurface)&   S1, 
-			   const Handle(Adaptor3d_HSurface)&   S2, 
+			   const Handle(Adaptor3d_Surface)&   S1, 
+			   const Handle(Adaptor3d_Surface)&   S2, 
 			   const Standard_Integer            iedge,
 			   const Standard_Boolean            isfirst,
 			   const Standard_Integer            cntlFiOnS)
@@ -400,17 +372,17 @@ void ChFi3d_Builder::Trunc(const Handle(ChFiDS_SurfData)&    SD,
   if(dis1 >= 0.1*dis2) tron = Standard_True;
   Standard_Integer ivois;
   if(!tron && YaUnVoisin(Spine,iedge,ivois,isfirst)) {
-    Handle(BRepAdaptor_HSurface) BS1 = Handle(BRepAdaptor_HSurface)::DownCast(S1);
-    Handle(BRepAdaptor_HSurface) BS2 = Handle(BRepAdaptor_HSurface)::DownCast(S2);
+    Handle(BRepAdaptor_Surface) BS1 = Handle(BRepAdaptor_Surface)::DownCast(S1);
+    Handle(BRepAdaptor_Surface) BS2 = Handle(BRepAdaptor_Surface)::DownCast(S2);
     if(!BS1.IsNull() && !BS2.IsNull()) {
       TopoDS_Face FBID;
-      TopoDS_Face F1 = BS1->ChangeSurface().Face();
-      TopoDS_Face F2 = BS2->ChangeSurface().Face();
+      TopoDS_Face F1 = BS1->Face();
+      TopoDS_Face F2 = BS2->Face();
       const ChFiDS_CommonPoint& cp1 = SD->Vertex(isfirst,1);
       const ChFiDS_CommonPoint& cp2 = SD->Vertex(isfirst,2);
       if(!((cp1.IsOnArc() && SearchFace(Spine,cp1,F1,FBID)) ||
            (cp2.IsOnArc() && SearchFace(Spine,cp2,F2,FBID)))) { 
-        tron = ChFi3d_KParticular(Spine,ivois,BS1->ChangeSurface(),BS2->ChangeSurface());
+        tron = ChFi3d_KParticular (Spine, ivois, *BS1, *BS2);
       }
     }
   }
@@ -534,7 +506,7 @@ static Standard_Boolean Tri(const Geom2dHatch_Hatcher& H,
   if (iSansFirst != 0) {
     if (iSansLast == 0) {
 #ifdef OCCT_DEBUG
-      cout<<"Parsing : Pb of Hatcher"<<endl;
+      std::cout<<"Parsing : Pb of Hatcher"<<std::endl;
 #endif
       return 0;
     }
@@ -603,9 +575,9 @@ static void FillSD (TopOpeBRepDS_DataStructure& DStr,
       for(;trouve;) {
 	const HatchGen_PointOnElement& PEtemp = pPH->Point(LeType);
 	IE = PEtemp.Index();
-	Handle(BRepAdaptor_HCurve2d) HE = Handle(BRepAdaptor_HCurve2d)::DownCast(M(IE));
+	Handle(BRepAdaptor_Curve2d) HE = Handle(BRepAdaptor_Curve2d)::DownCast(M(IE));
 	if(!HE.IsNull()) {
-	  const TopoDS_Edge& Etemp = HE->ChangeCurve2d().Edge();
+	  const TopoDS_Edge& Etemp = HE->Edge();
 	  TopExp::Vertices(Etemp,V1,V2);
 	}
 	else {
@@ -628,10 +600,10 @@ static void FillSD (TopOpeBRepDS_DataStructure& DStr,
     }
     const HatchGen_PointOnElement& PE = pPH->Point(LeType);
     Standard_Integer IE = PE.Index();
-    Handle(BRepAdaptor_HCurve2d) 
-      HE = Handle(BRepAdaptor_HCurve2d)::DownCast(M(IE));
+    Handle(BRepAdaptor_Curve2d) 
+      HE = Handle(BRepAdaptor_Curve2d)::DownCast(M(IE));
     if(HE.IsNull()) return;
-    const TopoDS_Edge& E = HE->ChangeCurve2d().Edge();
+    const TopoDS_Edge& E = HE->Edge();
     
     if (PE.Position() != TopAbs_INTERNAL) {
       TopAbs_Orientation O = CD->Interference(ons).Transition();
@@ -665,14 +637,14 @@ Standard_Boolean ChFi3d_Builder::SplitKPart
    ChFiDS_SequenceOfSurfData&       SetData, 
    const Handle(ChFiDS_Spine)&      Spine,
    const Standard_Integer           Iedge,
-   const Handle(Adaptor3d_HSurface)&  S1, 
+   const Handle(Adaptor3d_Surface)&  S1, 
    const Handle(Adaptor3d_TopolTool)& I1, 
-   const Handle(Adaptor3d_HSurface)&  S2,
+   const Handle(Adaptor3d_Surface)&  S2,
    const Handle(Adaptor3d_TopolTool)& I2, 
    Standard_Boolean&                intf,
    Standard_Boolean&                intl)
 {
-  //The the hatching of each faces is started by tangency lines.
+  //The hatching of each faces is started by tangency lines.
   
   Standard_Real pitol = Precision::PIntersection();
   
@@ -689,13 +661,12 @@ Standard_Boolean ChFi3d_Builder::SplitKPart
   if (!C1.IsNull()) {
     ll1.Load(C1);
     for(I1->Init(); I1->More(); I1->Next()) {
-      Handle(BRepAdaptor_HCurve2d) 
-	Bc = Handle(BRepAdaptor_HCurve2d)::DownCast(I1->Value());
-      Handle(Geom2dAdaptor_HCurve) 
-	Gc = Handle(Geom2dAdaptor_HCurve)::DownCast(I1->Value());
-      if(Bc.IsNull()) ie = H1.AddElement(Gc->ChangeCurve2d(),TopAbs_FORWARD);
-      else ie = H1.AddElement(Bc->ChangeCurve2d(),
-			      Bc->ChangeCurve2d().Edge().Orientation());
+      Handle(BRepAdaptor_Curve2d) 
+	Bc = Handle(BRepAdaptor_Curve2d)::DownCast(I1->Value());
+      Handle(Geom2dAdaptor_Curve) 
+	Gc = Handle(Geom2dAdaptor_Curve)::DownCast(I1->Value());
+      if(Bc.IsNull()) ie = H1.AddElement (*Gc, TopAbs_FORWARD);
+      else ie = H1.AddElement (*Bc, Bc->Edge().Orientation());
       M1.Bind(ie,I1->Value());
     }
     iH1 = H1.Trim(ll1);
@@ -704,7 +675,7 @@ Standard_Boolean ChFi3d_Builder::SplitKPart
     Nb1 = H1.NbDomains(iH1);
     if(Nb1 == 0) {
 #ifdef OCCT_DEBUG
-      cout<<"SplitKPart : tangency line out of the face"<<endl;
+      std::cout<<"SplitKPart : tangency line out of the face"<<std::endl;
 #endif
       return Standard_False;
     }
@@ -715,13 +686,12 @@ Standard_Boolean ChFi3d_Builder::SplitKPart
   if (!C2.IsNull()) {
     ll2.Load(C2);
     for(I2->Init(); I2->More(); I2->Next()) {
-      Handle(BRepAdaptor_HCurve2d) 
-	Bc = Handle(BRepAdaptor_HCurve2d)::DownCast(I2->Value());
-      Handle(Geom2dAdaptor_HCurve) 
-	Gc = Handle(Geom2dAdaptor_HCurve)::DownCast(I2->Value());
-      if(Bc.IsNull()) ie = H2.AddElement(Gc->ChangeCurve2d(),TopAbs_FORWARD);
-      else ie = H2.AddElement(Bc->ChangeCurve2d(),
-			      Bc->ChangeCurve2d().Edge().Orientation());
+      Handle(BRepAdaptor_Curve2d) 
+	Bc = Handle(BRepAdaptor_Curve2d)::DownCast(I2->Value());
+      Handle(Geom2dAdaptor_Curve) 
+	Gc = Handle(Geom2dAdaptor_Curve)::DownCast(I2->Value());
+      if(Bc.IsNull()) ie = H2.AddElement (*Gc, TopAbs_FORWARD);
+      else ie = H2.AddElement (*Bc, Bc->Edge().Orientation());
       M2.Bind(ie,I2->Value());
     }
     iH2 = H2.Trim(ll2);
@@ -730,7 +700,7 @@ Standard_Boolean ChFi3d_Builder::SplitKPart
     Nb2 = H2.NbDomains(iH2);
     if(Nb2 == 0) {
 #ifdef OCCT_DEBUG
-      cout<<"SplitKPart : tangency line out of the face"<<endl;
+      std::cout<<"SplitKPart : tangency line out of the face"<<std::endl;
 #endif
       return Standard_False;
     }
@@ -749,11 +719,11 @@ Standard_Boolean ChFi3d_Builder::SplitKPart
   
   // Return faces.
   TopoDS_Face F1, F2;
-  Handle(BRepAdaptor_HSurface) 
-    bhs = Handle(BRepAdaptor_HSurface)::DownCast(S1);
-  if(!bhs.IsNull()) F1 = bhs->ChangeSurface().Face();
-  bhs = Handle(BRepAdaptor_HSurface)::DownCast(S2);
-  if(!bhs.IsNull()) F2 = bhs->ChangeSurface().Face();
+  Handle(BRepAdaptor_Surface) 
+    bhs = Handle(BRepAdaptor_Surface)::DownCast(S1);
+  if(!bhs.IsNull()) F1 = bhs->Face();
+  bhs = Handle(BRepAdaptor_Surface)::DownCast(S2);
+  if(!bhs.IsNull()) F2 = bhs->Face();
   TopoDS_Face FBID;
   
   // Restriction of SurfDatas by cut lines.
@@ -772,7 +742,7 @@ Standard_Boolean ChFi3d_Builder::SplitKPart
   
   if (C1.IsNull() && C2.IsNull()) {
 #ifdef OCCT_DEBUG
-    cout<<"SplitData : 2 zero lines hatching impossible"<<endl;
+    std::cout<<"SplitData : 2 zero lines hatching impossible"<<std::endl;
 #endif
     return Standard_False;
   }
@@ -995,7 +965,7 @@ Standard_Boolean ChFi3d_Builder::SplitKPart
 	  onS = 2;
 #ifdef OCCT_DEBUG	
 	if (threeE[0].IsSame(threeE[1]))
-	  cout << "SplitKPart(), wrong corner vertex at switcher search" << endl;
+	  std::cout << "SplitKPart(), wrong corner vertex at switcher search" << std::endl;
 #endif
 	cntlFiOnS = 3 - onS;
       }
@@ -1090,7 +1060,7 @@ Standard_Boolean ChFi3d_Builder::SplitKPart
           onS = 2;
 #ifdef OCCT_DEBUG
         if (threeE[0].IsSame(threeE[1]))
-          cout << "SplitKPart(), wrong corner vertex at switcher search" << endl;
+          std::cout << "SplitKPart(), wrong corner vertex at switcher search" << std::endl;
 #endif
 	cntlFiOnS = 3 - onS;
       }
