@@ -24,13 +24,14 @@
 #include <MAT_ListOfEdge.hxx>
 #include <Precision.hxx>
 #include <TColStd_Array1OfInteger.hxx>
-#include <TColStd_DataMapOfIntegerInteger.hxx>
 
 //========================================================================
 //  function : MAT2d_Mat2d
 //  purpose  :
 //========================================================================
 MAT2d_Mat2d::MAT2d_Mat2d(const Standard_Boolean IsOpenResult)
+: semiInfinite(Standard_False),
+  isDone(Standard_False)
 {
   myIsOpenResult = IsOpenResult;
   thenumberofbisectors = 0;
@@ -183,6 +184,7 @@ void MAT2d_Mat2d::CreateMatOpen(MAT2d_Tool2d& atool)
 // du contour.
 // --------------------------------------------------------------------
   theedgelist = new MAT_ListOfEdge();
+  RemovedEdgesList = new MAT_ListOfEdge();
 
   for(i=0; i<noofedges; i++) {
     edge = new MAT_Edge();
@@ -401,7 +403,8 @@ void MAT2d_Mat2d::CreateMatOpen(MAT2d_Tool2d& atool)
 	    previousedge = currentedge;
 	    currentbisectorlist->Next();
 	  }
-	  
+
+          RemovedEdgesList->BackAdd(theedgelist->Current());
 	  theedgelist->Unlink();
 
 	  //-----------------------------------------------------------
@@ -623,6 +626,7 @@ void MAT2d_Mat2d::CreateMatOpen(MAT2d_Tool2d& atool)
 						 ->FirstBisector());
 
       for(j=0; j<noofarea(i); j++) {
+        RemovedEdgesList->BackAdd(theedgelist->Current());
 	theedgelist->Unlink();
 	theedgelist->Next();
 	shift++;
@@ -851,6 +855,7 @@ void MAT2d_Mat2d::CreateMat(MAT2d_Tool2d& atool)
   // du contour.
   // --------------------------------------------------------------------
   theedgelist = new MAT_ListOfEdge();
+  RemovedEdgesList = new MAT_ListOfEdge();
 
   for(i=0; i<noofedges; i++) {
     edge = new MAT_Edge();
@@ -1103,6 +1108,7 @@ void MAT2d_Mat2d::CreateMat(MAT2d_Tool2d& atool)
             currentbisectorlist->Next();
           }
 
+          RemovedEdgesList->BackAdd(theedgelist->Current());
           theedgelist->Unlink();
 
           //-----------------------------------------------------------
@@ -1329,6 +1335,7 @@ void MAT2d_Mat2d::CreateMat(MAT2d_Tool2d& atool)
         ->FirstBisector());
 
       for(j=0; j<noofarea(i); j++) {
+        RemovedEdgesList->BackAdd(theedgelist->Current());
         theedgelist->Unlink();
         theedgelist->Next();
         shift++;
@@ -1710,3 +1717,41 @@ Standard_Boolean MAT2d_Mat2d::IsDone() const
   return isDone;
 }
 
+//=======================================================================
+//function : ~MAT2d_Mat2d
+//purpose  : 
+//=======================================================================
+
+MAT2d_Mat2d::~MAT2d_Mat2d()
+{
+  MAT_DataMapIteratorOfDataMapOfIntegerBisector itmap(bisectormap);
+  for (; itmap.More(); itmap.Next())
+  {
+    Handle(MAT_Bisector) aBisector = itmap.Value();
+    aBisector->FirstEdge(NULL);
+    aBisector->SecondEdge(NULL);
+  }
+
+  if (!theedgelist.IsNull())
+  {
+    theedgelist->First();
+    for (Standard_Integer i = 1; i <= theedgelist->Number(); i++)
+    {
+      Handle(MAT_Edge) anEdge = theedgelist->Current();
+      anEdge->FirstBisector(NULL);
+      anEdge->SecondBisector(NULL);
+      theedgelist->Next();
+    }
+  }
+  if (!RemovedEdgesList.IsNull())
+  {
+    RemovedEdgesList->First();
+    for (Standard_Integer i = 1; i <= RemovedEdgesList->Number(); i++)
+    {
+      Handle(MAT_Edge) anEdge = RemovedEdgesList->Current();
+      anEdge->FirstBisector(NULL);
+      anEdge->SecondBisector(NULL);
+      RemovedEdgesList->Next();
+    }
+  }
+}
