@@ -13,19 +13,17 @@
 // Alternatively, this file may be used under the terms of Open CASCADE
 // commercial license or contractual agreement.
 
-#include <OpenGl_Aspects.hxx>
 #include <OpenGl_GlCore11.hxx>
 #include <OpenGl_GraphicDriver.hxx>
 #include <OpenGl_ShaderManager.hxx>
 #include <OpenGl_ShaderProgram.hxx>
-#include <OpenGl_ShaderStates.hxx>
 #include <OpenGl_Text.hxx>
-#include <OpenGl_Workspace.hxx>
 #include <OpenGl_View.hxx>
 #include <OpenGl_VertexBufferCompat.hxx>
 
 #include <Font_FontMgr.hxx>
 #include <Font_FTFont.hxx>
+#include <Font_TextFormatter.hxx>
 #include <Graphic3d_TransformUtils.hxx>
 #include <TCollection_HAsciiString.hxx>
 
@@ -206,6 +204,52 @@ void OpenGl_Text::Release (OpenGl_Context* theCtx)
   }
 }
 
+// =======================================================================
+// function : EstimatedDataSize
+// purpose  :
+// =======================================================================
+Standard_Size OpenGl_Text::EstimatedDataSize() const
+{
+  Standard_Size aSize = 0;
+  for (Standard_Integer anIter = myVertsVbo.Lower(); anIter <= myVertsVbo.Upper(); ++anIter)
+  {
+    if (const Handle(OpenGl_VertexBuffer)& aVerts = myVertsVbo.Value (anIter))
+    {
+      aSize += aVerts->EstimatedDataSize();
+    }
+    if (const Handle(OpenGl_VertexBuffer)& aTCrds = myTCrdsVbo.Value (anIter))
+    {
+      aSize += aTCrds->EstimatedDataSize();
+    }
+  }
+  if (!myBndVertsVbo.IsNull())
+  {
+    aSize += myBndVertsVbo->EstimatedDataSize();
+  }
+  return aSize;
+}
+
+// =======================================================================
+// function : UpdateDrawStats
+// purpose  :
+// =======================================================================
+void OpenGl_Text::UpdateDrawStats (Graphic3d_FrameStatsDataTmp& theStats,
+                                   bool theIsDetailed) const
+{
+  ++theStats[Graphic3d_FrameStatsCounter_NbElemsNotCulled];
+  ++theStats[Graphic3d_FrameStatsCounter_NbElemsTextNotCulled];
+  if (theIsDetailed)
+  {
+    for (Standard_Integer anIter = myVertsVbo.Lower(); anIter <= myVertsVbo.Upper(); ++anIter)
+    {
+      if (const Handle(OpenGl_VertexBuffer)& aVerts = myVertsVbo.Value (anIter))
+      {
+        theStats[Graphic3d_FrameStatsCounter_NbTrianglesNotCulled] += aVerts->GetElemsNb() / 3; // 2 non-indexed triangles per glyph
+      }
+    }
+  }
+}
+
 // =======================================================================
 // function : StringSize
 // purpose  :
@@ -215,6 +259,7 @@ void OpenGl_Text::StringSize (const Handle(OpenGl_Context)& theCtx,
                               const OpenGl_Aspects&         theTextAspect,
                               const Standard_ShortReal      theHeight,
                               const unsigned int            theResolution,
+                              const Font_Hinting            theFontHinting,
                               Standard_ShortReal&           theWidth,
                               Standard_ShortReal&           theAscent,
                               Standard_ShortReal&           theDescent)
@@ -222,8 +267,8 @@ void OpenGl_Text::StringSize (const Handle(OpenGl_Context)& theCtx,
   theWidth   = 0.0f;
   theAscent  = 0.0f;
   theDescent = 0.0f;
-  const TCollection_AsciiString aFontKey = FontKey (theTextAspect, (Standard_Integer)theHeight, theResolution);
-  Handle(OpenGl_Font) aFont = FindFont (theCtx, theTextAspect, (Standard_Integer)theHeight, theResolution, aFontKey);
+  const TCollection_AsciiString aFontKey = FontKey (theTextAspect, (Standard_Integer)theHeight, theResolution, theFontHinting);
+  Handle(OpenGl_Font) aFont = FindFont (theCtx, theTextAspect, (Standard_Integer)theHeight, theResolution, theFontHinting, aFontKey);
   if (aFont.IsNull() || !aFont->IsValid())
   {
     return;
@@ -278,16 +323,16 @@ void OpenGl_Text::StringSize (const Handle(OpenGl_Context)& theCtx,
 // =======================================================================
 void OpenGl_Text::Render (const Handle(OpenGl_Workspace)& theWorkspace) const
 {
-  const OpenGl_Aspects* aTextAspect = theWorkspace->ApplyAspects();
+  const OpenGl_Aspects* aTextAspect = theWorkspace->ApplyAspects (false); // do not bind textures as they will be disabled anyway
   const Handle(OpenGl_Context)& aCtx = theWorkspace->GetGlContext();
-  const Handle(OpenGl_TextureSet) aPrevTexture = aCtx->BindTextures (Handle(OpenGl_TextureSet)());
 
   // Bind custom shader program or generate default version
   aCtx->ShaderManager()->BindFontProgram (aTextAspect->ShaderProgramRes (aCtx));
+  const Handle(OpenGl_TextureSet) aPrevTexture = aCtx->BindTextures (Handle(OpenGl_TextureSet)(), Handle(OpenGl_ShaderProgram)());
 
   if (myText->HasPlane() && myText->HasOwnAnchorPoint())
   {
-    myOrientationMatrix = theWorkspace->View()->Camera()->OrientationMatrix();
+    myOrientationMatrix = aCtx->Camera()->OrientationMatrix();
     // reset translation part
     myOrientationMatrix.ChangeValue (0, 3) = 0.0;
     myOrientationMatrix.ChangeValue (1, 3) = 0.0;
@@ -301,18 +346,19 @@ void OpenGl_Text::Render (const Handle(OpenGl_Workspace)& theWorkspace) const
           *aTextAspect,
           theWorkspace->TextColor(),
           theWorkspace->TextSubtitleColor(),
-          aCtx->Resolution());
+          aCtx->Resolution(),
+          theWorkspace->View()->RenderingParams().FontHinting);
 
   // restore aspects
   if (!aPrevTexture.IsNull())
   {
-    aCtx->BindTextures (aPrevTexture);
+    aCtx->BindTextures (aPrevTexture, Handle(OpenGl_ShaderProgram)());
   }
 
   // restore Z buffer settings
   if (theWorkspace->UseZBuffer())
   {
-    glEnable (GL_DEPTH_TEST);
+    aCtx->core11fwd->glEnable (GL_DEPTH_TEST);
   }
 }
 
@@ -322,22 +368,20 @@ void OpenGl_Text::Render (const Handle(OpenGl_Workspace)& theWorkspace) const
 // =======================================================================
 void OpenGl_Text::Render (const Handle(OpenGl_Context)& theCtx,
                           const OpenGl_Aspects& theTextAspect,
-                          unsigned int theResolution) const
+                          unsigned int theResolution,
+                          Font_Hinting theFontHinting) const
 {
-#if !defined(GL_ES_VERSION_2_0)
   const Standard_Integer aPrevPolygonMode  = theCtx->SetPolygonMode (GL_FILL);
   const bool             aPrevHatchingMode = theCtx->SetPolygonHatchEnabled (false);
-#endif
 
   render (theCtx, theTextAspect,
           theTextAspect.Aspect()->ColorRGBA(),
           theTextAspect.Aspect()->ColorSubTitleRGBA(),
-          theResolution);
+          theResolution,
+          theFontHinting);
 
-#if !defined(GL_ES_VERSION_2_0)
   theCtx->SetPolygonMode         (aPrevPolygonMode);
   theCtx->SetPolygonHatchEnabled (aPrevHatchingMode);
-#endif
 }
 
 // =======================================================================
@@ -426,6 +470,12 @@ void OpenGl_Text::setupMatrix (const Handle(OpenGl_Context)& theCtx,
   {
     OpenGl_Mat4d aCurrentWorldViewMat;
     aCurrentWorldViewMat.Convert (theCtx->WorldViewState.Current());
+
+    // apply local transformation
+    OpenGl_Mat4d aModelWorld;
+    aModelWorld.Convert (theCtx->ModelWorldState.Current());
+    aCurrentWorldViewMat = aCurrentWorldViewMat * aModelWorld;
+
     theCtx->WorldViewState.SetCurrent<Standard_Real> (aCurrentWorldViewMat * aModViewMat);
   }
   else
@@ -461,19 +511,19 @@ void OpenGl_Text::drawText (const Handle(OpenGl_Context)& theCtx,
   for (Standard_Integer anIter = 0; anIter < myTextures.Length(); ++anIter)
   {
     const GLuint aTexId = myTextures.Value (anIter);
-    glBindTexture (GL_TEXTURE_2D, aTexId);
+    theCtx->core11fwd->glBindTexture (GL_TEXTURE_2D, aTexId);
 
     const Handle(OpenGl_VertexBuffer)& aVerts = myVertsVbo.Value (anIter);
     const Handle(OpenGl_VertexBuffer)& aTCrds = myTCrdsVbo.Value (anIter);
     aVerts->BindAttribute (theCtx, Graphic3d_TOA_POS);
     aTCrds->BindAttribute (theCtx, Graphic3d_TOA_UV);
 
-    glDrawArrays (GL_TRIANGLES, 0, GLsizei(aVerts->GetElemsNb()));
+    theCtx->core11fwd->glDrawArrays (GL_TRIANGLES, 0, GLsizei(aVerts->GetElemsNb()));
 
     aTCrds->UnbindAttribute (theCtx, Graphic3d_TOA_UV);
     aVerts->UnbindAttribute (theCtx, Graphic3d_TOA_POS);
   }
-  glBindTexture (GL_TEXTURE_2D, 0);
+  theCtx->core11fwd->glBindTexture (GL_TEXTURE_2D, 0);
 }
 
 // =======================================================================
@@ -482,16 +532,21 @@ void OpenGl_Text::drawText (const Handle(OpenGl_Context)& theCtx,
 // =======================================================================
 TCollection_AsciiString OpenGl_Text::FontKey (const OpenGl_Aspects& theAspect,
                                               Standard_Integer theHeight,
-                                              unsigned int theResolution)
+                                              unsigned int theResolution,
+                                              Font_Hinting theFontHinting)
 {
   const Font_FontAspect anAspect = theAspect.Aspect()->TextFontAspect() != Font_FA_Undefined
                                  ? theAspect.Aspect()->TextFontAspect()
                                  : Font_FA_Regular;
   const TCollection_AsciiString& aFont = !theAspect.Aspect()->TextFont().IsNull() ? theAspect.Aspect()->TextFont()->String() : THE_DEFAULT_FONT;
-  return aFont
-       + TCollection_AsciiString(":") + Standard_Integer(anAspect)
-       + TCollection_AsciiString(":") + Standard_Integer(theResolution)
-       + TCollection_AsciiString(":") + theHeight;
+
+  char aSuff[64];
+  Sprintf (aSuff, ":%d:%d:%d:%d",
+           Standard_Integer(anAspect),
+           Standard_Integer(theResolution),
+           theHeight,
+           Standard_Integer(theFontHinting));
+  return aFont + aSuff;
 }
 
 // =======================================================================
@@ -502,6 +557,7 @@ Handle(OpenGl_Font) OpenGl_Text::FindFont (const Handle(OpenGl_Context)& theCtx,
                                            const OpenGl_Aspects& theAspect,
                                            Standard_Integer theHeight,
                                            unsigned int theResolution,
+                                           Font_Hinting theFontHinting,
                                            const TCollection_AsciiString& theKey)
 {
   Handle(OpenGl_Font) aFont;
@@ -522,6 +578,7 @@ Handle(OpenGl_Font) OpenGl_Text::FindFont (const Handle(OpenGl_Context)& theCtx,
     Font_FTFontParams aParams;
     aParams.PointSize  = theHeight;
     aParams.Resolution = theResolution;
+    aParams.FontHinting = theFontHinting;
     if (Handle(Font_FTFont) aFontFt = Font_FTFont::FindAndCreate (aFontName, anAspect, aParams, Font_StrictLevel_Any))
     {
       aFont = new OpenGl_Font (aFontFt, theKey);
@@ -576,17 +633,16 @@ void OpenGl_Text::drawRect (const Handle(OpenGl_Context)& theCtx,
   }
 
   // bind unlit program
-  theCtx->ShaderManager()->BindFaceProgram (Handle(OpenGl_TextureSet)(), Graphic3d_TOSM_UNLIT,
+  theCtx->ShaderManager()->BindFaceProgram (Handle(OpenGl_TextureSet)(), Graphic3d_TypeOfShadingModel_Unlit,
                                             Graphic3d_AlphaMode_Opaque, Standard_False, Standard_False,
                                             Handle(OpenGl_ShaderProgram)());
 
-#if !defined(GL_ES_VERSION_2_0)
-  if (theCtx->core11 != NULL
+  if (theCtx->core11ffp != NULL
    && theCtx->ActiveProgram().IsNull())
   {
-    glBindTexture (GL_TEXTURE_2D, 0);
+    theCtx->core11fwd->glBindTexture (GL_TEXTURE_2D, 0);
   }
-#endif
+
   theCtx->SetColor4fv (theColorSubs);
   setupMatrix (theCtx, theTextAspect, OpenGl_Vec3 (0.0f, 0.0f, 0.0f));
   myBndVertsVbo->BindAttribute (theCtx, Graphic3d_TOA_POS);
@@ -605,16 +661,17 @@ void OpenGl_Text::render (const Handle(OpenGl_Context)& theCtx,
                           const OpenGl_Aspects& theTextAspect,
                           const OpenGl_Vec4& theColorText,
                           const OpenGl_Vec4& theColorSubs,
-                          unsigned int theResolution) const
+                          unsigned int theResolution,
+                          Font_Hinting theFontHinting) const
 {
-  if (myText->Text().IsEmpty())
+  if (myText->Text().IsEmpty() && myText->TextFormatter().IsNull())
   {
     return;
   }
 
   // Note that using difference resolution in different Views in same Viewer
   // will lead to performance regression (for example, text will be recreated every time).
-  const TCollection_AsciiString aFontKey = FontKey (theTextAspect, (Standard_Integer)myText->Height(), theResolution);
+  const TCollection_AsciiString aFontKey = FontKey (theTextAspect, (Standard_Integer)myText->Height(), theResolution, theFontHinting);
   if (!myFont.IsNull()
    && !myFont->ResourceKey().IsEqual (aFontKey))
   {
@@ -624,22 +681,26 @@ void OpenGl_Text::render (const Handle(OpenGl_Context)& theCtx,
 
   if (myFont.IsNull())
   {
-    myFont = FindFont (theCtx, theTextAspect, (Standard_Integer)myText->Height(), theResolution, aFontKey);
+    myFont = FindFont (theCtx, theTextAspect, (Standard_Integer)myText->Height(), theResolution, theFontHinting, aFontKey);
   }
-  if (!myFont->WasInitialized())
+  if (myFont.IsNull()
+  || !myFont->WasInitialized())
   {
     return;
   }
 
   if (myTextures.IsEmpty())
   {
-    Font_TextFormatter aFormatter;
+    Handle(Font_TextFormatter) aFormatter = myText->TextFormatter();
+    if (aFormatter.IsNull())
+    {
+      aFormatter = new Font_TextFormatter();
+    }
+    aFormatter->SetupAlignment (myText->HorizontalAlignment(), myText->VerticalAlignment());
+    aFormatter->Reset();
 
-    aFormatter.SetupAlignment (myText->HorizontalAlignment(), myText->VerticalAlignment());
-    aFormatter.Reset();
-
-    aFormatter.Append (myText->Text(), *myFont->FTFont());
-    aFormatter.Format();
+    aFormatter->Append (myText->Text(), *myFont->FTFont());
+    aFormatter->Format();
 
     OpenGl_TextBuilder aBuilder;
     aBuilder.Perform (aFormatter,
@@ -649,7 +710,7 @@ void OpenGl_Text::render (const Handle(OpenGl_Context)& theCtx,
                       myVertsVbo,
                       myTCrdsVbo);
 
-    aFormatter.BndBox (myBndBox);
+    aFormatter->BndBox (myBndBox);
     if (!myBndVertsVbo.IsNull())
     {
       myBndVertsVbo->Release (theCtx.get());
@@ -689,44 +750,38 @@ void OpenGl_Text::render (const Handle(OpenGl_Context)& theCtx,
     }
   }
 
-#if !defined(GL_ES_VERSION_2_0)
-  if (theCtx->core11 != NULL
+  if (theCtx->core11ffp != NULL
    && theCtx->caps->ffpEnable)
   {
-    glDisable (GL_LIGHTING);
+    theCtx->core11fwd->glDisable (GL_LIGHTING);
   }
-#endif
 
   // setup depth test
   const bool hasDepthTest = !myIs2d
                          && theTextAspect.Aspect()->TextStyle() != Aspect_TOST_ANNOTATION;
   if (!hasDepthTest)
   {
-    glDisable (GL_DEPTH_TEST);
+    theCtx->core11fwd->glDisable (GL_DEPTH_TEST);
   }
 
   if (theCtx->core15fwd != NULL)
   {
     theCtx->core15fwd->glActiveTexture (GL_TEXTURE0);
   }
-#if !defined(GL_ES_VERSION_2_0)
+
   // activate texture unit
-  GLint aTexEnvParam = GL_REPLACE;
-  if (theCtx->core11 != NULL)
+  if (theCtx->core11ffp != NULL && theCtx->ActiveProgram().IsNull())
   {
-    glDisable (GL_TEXTURE_1D);
-    glEnable  (GL_TEXTURE_2D);
-    glGetTexEnviv (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, &aTexEnvParam);
-    if (aTexEnvParam != GL_REPLACE)
-    {
-      glTexEnvi (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
-    }
+    const Handle(OpenGl_Texture)& aTexture = myFont->Texture();
+    OpenGl_Sampler::applyGlobalTextureParams (theCtx, *aTexture, aTexture->Sampler()->Parameters());
   }
-#endif
 
   // setup blending
-  glEnable (GL_BLEND);
-  glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+  if (theTextAspect.Aspect()->AlphaMode() == Graphic3d_AlphaMode_MaskBlend)
+  {
+    theCtx->core11fwd->glEnable (GL_BLEND);
+    theCtx->core11fwd->glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+  }
 
   // alpha to coverage makes text too thin
   theCtx->SetSampleAlphaToCoverage (false);
@@ -736,10 +791,11 @@ void OpenGl_Text::render (const Handle(OpenGl_Context)& theCtx,
   {
     case Aspect_TODT_BLEND:
     {
-    #if !defined(GL_ES_VERSION_2_0)
-      glEnable  (GL_COLOR_LOGIC_OP);
-      glLogicOp (GL_XOR);
-    #endif
+      if (theCtx->GraphicsLibrary() == Aspect_GraphicsLibrary_OpenGL)
+      {
+        theCtx->core11fwd->glEnable (GL_COLOR_LOGIC_OP);
+        theCtx->core11fwd->glLogicOp (GL_XOR);
+      }
       break;
     }
     case Aspect_TODT_SUBTITLE:
@@ -788,48 +844,73 @@ void OpenGl_Text::render (const Handle(OpenGl_Context)& theCtx,
     theCtx->ApplyProjectionMatrix();
   }
 
-#if !defined(GL_ES_VERSION_2_0)
-  if (theCtx->core11 != NULL)
+  if (theCtx->core11ffp != NULL && theCtx->ActiveProgram().IsNull())
   {
-    glTexEnvi (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, aTexEnvParam);
+    const Handle(OpenGl_Texture)& aTexture = myFont->Texture();
+    OpenGl_Sampler::resetGlobalTextureParams (theCtx, *aTexture, aTexture->Sampler()->Parameters());
   }
-#endif
 
   if (theTextAspect.Aspect()->TextDisplayType() == Aspect_TODT_DIMENSION)
   {
-    glDisable (GL_BLEND);
+    if (theTextAspect.Aspect()->AlphaMode() == Graphic3d_AlphaMode_MaskBlend)
+    {
+      theCtx->core11fwd->glDisable (GL_BLEND);
+    }
     if (!myIs2d)
     {
-      glDisable (GL_DEPTH_TEST);
+      theCtx->core11fwd->glDisable (GL_DEPTH_TEST);
     }
-  #if !defined(GL_ES_VERSION_2_0)
-    if (theCtx->core11 != NULL)
+    if (theCtx->core11ffp != NULL)
     {
-      glDisable (GL_TEXTURE_2D);
+      theCtx->core11fwd->glDisable (GL_TEXTURE_2D);
     }
-  #endif
+
     const bool aColorMaskBack = theCtx->SetColorMask (false);
 
-    glClear (GL_STENCIL_BUFFER_BIT);
-    glEnable (GL_STENCIL_TEST);
-    glStencilFunc (GL_ALWAYS, 1, 0xFF);
-    glStencilOp (GL_KEEP, GL_KEEP, GL_REPLACE);
+    theCtx->core11fwd->glClear (GL_STENCIL_BUFFER_BIT);
+    theCtx->core11fwd->glEnable (GL_STENCIL_TEST);
+    theCtx->core11fwd->glStencilFunc (GL_ALWAYS, 1, 0xFF);
+    theCtx->core11fwd->glStencilOp (GL_KEEP, GL_KEEP, GL_REPLACE);
 
     drawRect (theCtx, theTextAspect, OpenGl_Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
 
-    glStencilFunc (GL_ALWAYS, 0, 0xFF);
+    theCtx->core11fwd->glStencilFunc (GL_ALWAYS, 0, 0xFF);
 
     theCtx->SetColorMask (aColorMaskBack);
   }
 
   // reset OpenGL state
-  glDisable (GL_BLEND);
-  glDisable (GL_STENCIL_TEST);
-#if !defined(GL_ES_VERSION_2_0)
-  glDisable (GL_COLOR_LOGIC_OP);
-#endif
+  if (theTextAspect.Aspect()->AlphaMode() == Graphic3d_AlphaMode_MaskBlend)
+  {
+    theCtx->core11fwd->glDisable (GL_BLEND);
+  }
+  theCtx->core11fwd->glDisable (GL_STENCIL_TEST);
+  if (theCtx->GraphicsLibrary() == Aspect_GraphicsLibrary_OpenGL)
+  {
+    theCtx->core11fwd->glDisable (GL_COLOR_LOGIC_OP);
+  }
 
   // model view matrix was modified
   theCtx->WorldViewState.Pop();
   theCtx->ApplyModelViewMatrix();
 }
+
+// =======================================================================
+// function : DumpJson
+// purpose  :
+// =======================================================================
+void OpenGl_Text::DumpJson (Standard_OStream& theOStream, Standard_Integer theDepth) const
+{
+  OCCT_DUMP_CLASS_BEGIN (theOStream, OpenGl_Text)
+  OCCT_DUMP_BASE_CLASS (theOStream, theDepth, OpenGl_Element)
+
+  OCCT_DUMP_FIELD_VALUE_NUMERICAL (theOStream, myTextures.Size())
+
+  for (NCollection_Vector<Handle(OpenGl_VertexBuffer)>::Iterator aCrdsIt (myTCrdsVbo); aCrdsIt.More(); aCrdsIt.Next())
+  {
+    const Handle(OpenGl_VertexBuffer)& aVertexBuffer = aCrdsIt.Value();
+    OCCT_DUMP_FIELD_VALUES_DUMPED (theOStream, theDepth, aVertexBuffer.get())
+  }
+
+  OCCT_DUMP_FIELD_VALUES_DUMPED (theOStream, theDepth, &myBndBox)
+}
